<?php
/**
 * Created by PhpStorm.
 * User: spookee
 * Date: 13.09.18
 * Time: 10:20
 */

namespace WS\EMarketing\Resource;

use WS\EMarketing\CaseConverter;

/**
 * Class AbstractResource
 * @package WS\EMarketing\Resource
 */
abstract class AbstractResource implements \JsonSerializable
{

    /**
     * @return array|mixed
     */
    public function jsonSerialize()
    {
        $data = [];
        foreach (get_object_vars($this) as $prop => $value) {
            if ($this->$prop !== null) {
                if ($value instanceof \DateTime) {
                    $data[self::fromCamelCase($prop)] = $value->format('Y-m-d');
                    continue;
                }
                //if (is_array($value) && !count($value)) {
                //    continue;
                //}

                if (is_string($value)) {
                    $value = utf8_encode($value);
                }

                $data[self::fromCamelCase($prop)] = $value;
            }
        }
        return $data;
    }

    /**
     * @param $key
     * @param $value
     * @param $errorMessage
     * @param bool $nullable
     * @return bool
     */
    protected function _isValidString($key, $value, &$errorMessage, $nullable = true)
    {
        if ($nullable === false) {
            if ($value === null || !strlen(trim($value))) {
                $errorMessage = "'{$key}' must be null or a string.";
                return false;
            }
            return true;
        }
        if ($value !== null && '' === trim($value)) {
            $errorMessage = "'{$key}' must be a string or null.";
            return false;
        }
        return true;
    }

    use CaseConverter;

    protected function _isValidNumber($key, $value, &$errorMessage, $nullable = true)
    {
        if ($nullable === false) {
            if ($value === null || !is_numeric($value)) {
                $errorMessage = "'{$key}' must be null or a number.";
                return false;
            }
            return true;
        }
        if ($value !== null && !is_numeric($value)) {
            $errorMessage = "'{$key}' must be a number.";
            return false;
        }
        return true;
    }

    /**
     * @param $key
     * @param $value
     * @param $errorMessage
     * @param bool $nullable
     * @return bool
     */
    protected function _isValidBool($key, $value, &$errorMessage, $nullable = true)
    {
        if ($nullable === false) {
            if ($value === null || ($value !== true || $value !== false)) {
                $errorMessage = "'{$key}' must be null or a boolean.";
                return false;
            }
            return true;
        }
        if ($value !== null && $value !== true && $value !== false) {
            $errorMessage = "'{$key}' must be a boolean.";
            return false;
        }
        return true;
    }

    /**
     * @param $key
     * @param $value
     * @param $errorMessage
     * @param bool $nullable
     * @param bool|array $set
     * @return bool
     */
    protected function _isValidArray($key, $value, &$errorMessage, $nullable = true)
    {
        if ($nullable === false && !is_array($value)) {
            $errorMessage = "'{$key}' must be an array.";
            return false;
        }

        if ($nullable !== true && $value !== null && !is_array($value)) {
            $errorMessage = "'{$key}' must be null or an array.";
            return false;
        }

        return true;
    }

    /**
     * @param $key
     * @param $value
     * @param $errorMessage
     * @param $set
     * @param bool $nullale
     * @return bool
     */
    protected function _isValidSet($key, $value, &$errorMessage, array $set, $nullale = true)
    {
        if (!$nullale && $value === null) {
            $errorMessage = "'{$key}' must be one of " . implode(', ', $set) . '.';
            return false;
        }
        if ($value !== null && !in_array($value, $set)) {
            $errorMessage = "'{$key}' must be one of " . implode(', ', $set) . '.';
            return false;
        }
        return true;
    }

    /**
     * @param $key
     * @param $value
     * @param $errorMessage
     * @param $pattern
     * @param bool $nullable
     * @return bool
     */
    protected function _isValidRegexString($key, $value, &$errorMessage, $pattern, $nullable = true, $format = false)
    {
        if ($nullable === false) {
            if ($value === null || !preg_match($pattern, $value)) {
                $errorMessage = "'{$key}' must be null or a valid string" . ($format ? " (Format: '{$format}')" : '') . ".";
                return false;
            }
            return true;
        }
        if ($value !== null && !preg_match($pattern, $value)) {
            $errorMessage = "'{$key}' must be a valid string" . ($format ? " (Format: '{$format}')" : '') . ".";
            return false;
        }
        return true;
    }

    /**
     * @param $key
     * @param $value
     * @param $class
     * @param $errorMessage
     * @param bool $nullable
     * @return bool
     */
    protected function _isValidObject($key, $value, $class, &$errorMessage, $nullable = true)
    {

        // TODO: is object valid?
        if ($nullable === false) {
            if (!($value instanceof $class)) {
                $errorMessage = "'{$key}' must be an instance of {$class}.";
                return false;
            }
            return true;
        }
        if ($value !== null) {
            if (!($value instanceof $class)) {
                $errorMessage = "'{$key}' must be null or an instance of {$class}.";
                return false;
            }
            if ($value instanceof AbstractResource) {
                return $value->isValid($errorMessage);
            }
        }
        return true;
    }

    /**
     * Checks if resource is valid
     *
     * @return bool
     */
    abstract public function isValid(&$errorMessage);
}